import matplotlib.pyplot as plt
import pandas as pd
import numpy as np
import os
from pathlib import Path
from dataclasses import dataclass

plt.style.use(os.path.join(os.path.dirname(__file__), "glumt.mplrc"))


def nestedSet(dic, keys, value):
    for key in keys[:-1]:
        dic = dic.setdefault(key, {})
    dic[keys[-1]] = value


def calcPosNegSums(df):
    posSum = pd.pivot_table(
        df,
        index=df['date'].dt.month,
        columns=df['date'].dt.year,
        values='value',
        aggfunc=lambda x: x[x > 0].sum()
    )
    negSum = pd.pivot_table(
        df,
        index=df['date'].dt.month,
        columns=df['date'].dt.year,
        values='value',
        aggfunc=lambda x: x[x < 0].sum()
    )
    posSum = posSum.rename_axis("month")
    posSum = posSum.rename_axis("year", axis="columns")
    negSum = negSum.rename_axis("month")
    negSum = negSum.rename_axis("year", axis="columns")
    negSum = negSum.apply(np.abs)
    return posSum, negSum


@dataclass
class PlotOptions:
    height: float
    width: float
    formats = ["pdf", "png"]


@dataclass
class Ecoplot:
    plotDir: str
    plotOptions = PlotOptions(3.14, 2.355)
    plotPaths = {"all": {}, "years": {}}

    def saveFig(self, fig, filename):

        fig.set_size_inches(self.plotOptions.height, self.plotOptions.width)
        fig.tight_layout()

        for filetype in self.plotOptions.formats:
            fig.savefig(f"{filename}.{filetype}", dpi=600)

    def plotTimeline(self, transactions):
        timeline = transactions[["date", "saldo"]]
        timeline = timeline.iloc[::-1]

        fig = plt.figure()
        fig.add_subplot(111)
        plt.step(x=timeline["date"], y=timeline["saldo"])
        plt.ylabel("Saldo / EUR")
        plt.xticks(rotation=45)

        filename = Path(self.plotDir) / "ecoTimeline"
        self.saveFig(fig, filename)
        plt.close(fig)

        self.plotPaths["all"].update({"timeline": filename})

    def plotPie(self, transactions):
        fig = plt.figure()
        ax = fig.add_subplot(111)
        d = transactions['groupID'].value_counts()
        # sum all neg and pos; subplot for both
        d = transactions.pivot_table(
            index=["groupID"], aggfunc={"value": lambda x: np.sum(np.abs(x))})
        d.plot.pie(y="value", figsize=(5, 5), ax=ax, legend=False)
        plt.ylabel("")

        filename = Path(self.plotDir) / "ecoPie"
        self.saveFig(fig, filename)
        plt.close(fig)

        self.plotPaths["all"].update({"pie": filename})

    def plotCategories(self, transactions):

        absGroupVal = transactions.pivot_table(
            values=["value"],
            index=["groupID"],
            aggfunc={"value": np.sum}
        )

        fig = plt.figure()
        ax = fig.add_subplot(111)
        absGroupVal.plot.barh(y="value", ax=ax, legend=False)
        plt.xlabel("summation / EUR")
        plt.ylabel("")
        filename = Path(self.plotDir) / "ecoNettoHbarTotal"
        self.saveFig(fig, filename)
        plt.close(fig)

        self.plotPaths["all"].update({"categories": filename})

    def plotBars(self, transactions):
        df = transactions
        posSum, negSum = calcPosNegSums(df)

        fig = plt.figure()
        ax = fig.add_subplot(111)
        yearDF = pd.concat([posSum.sum().rename('in'),
                            negSum.sum().rename('out')],
                           axis=1)
        yearDF.plot.bar(ax=ax)
        plt.ylabel("summation / EUR")
        plt.xticks(rotation=45)
        # ax.set_ylim([minSaldo*1.1, maxSaldo*1.1])
        filename = Path(self.plotDir) / "ecoYearTotal"
        self.saveFig(fig, filename)
        plt.close(fig)

        self.plotPaths["all"].update({"years": filename})

    def plotBarsYearly(self, transactions):
        """Show total in and out per month over a year"""
        df = transactions
        posSum, negSum = calcPosNegSums(df)

        minSaldo = negSum.min().min()
        maxSaldo = posSum.max().max()

        years = posSum.columns
        for y in years:
            fig = plt.figure()
            ax = fig.add_subplot(111)
            yearDF = pd.concat(
                [posSum.loc[:, y].rename(
                    'in'), negSum.loc[:, y].rename('out')],
                axis=1)
            yearDF.plot.bar(ax=ax)
            ax.set_ylim([minSaldo * 1.1, maxSaldo * 1.1])
            plt.ylabel("summation / EUR")
            filename = Path(self.plotDir) / f"ecoYearTest{y}"
            self.saveFig(fig, filename)
            plt.close(fig)

            nestedSet(self.plotPaths, ["years", f"{y}", "year"], filename)

    def plotCategoriesYearly(self, transactions):

        df = transactions
        yearTrans = pd.pivot_table(
            df,
            index=df['date'].dt.year,
            columns=df['groupID'],
            values='value',
            aggfunc=np.sum
        )

        years = yearTrans.index
        for y in years:
            fig = plt.figure()
            ax = fig.add_subplot(111)
            yearTrans.loc[y, :].plot.barh(y="value", ax=ax)
            plt.xlabel("summation / EUR")
            plt.ylabel("")
            filename = Path(self.plotDir) / f"ecoNettoHbar{y}"
            self.saveFig(fig, filename)
            plt.close(fig)

            nestedSet(self.plotPaths, [
                      "years", f"{y}", "categories"], filename)



import re

import pandas as pd


class BankAccount:

    dataframeCols = [
        "date",
        "valtua",
        "customer",
        "type",
        "usage",
        "saldo",
        "saldoCurrency",
        "value",
        "valueCurrency",
    ]

    def __init__(self, owner, accountNumber, bank, transactions, groupSettings):
        self.owner = owner
        self.accountNumber = accountNumber
        self.bank = bank
        self.transactions = transactions
        self.groupSettings = groupSettings

    def update(self, transactionDataframe):

        self.transactions = pd.concat(
            [self.transactions, transactionDataframe])

        self.transactions = self.transactions.sort_values(
            "date", ascending=False)
        self.transactions = self.transactions.reset_index(drop=True)

        self.transactions.drop_duplicates(
            subset=self.dataframeCols, inplace=True)

        self.groupTransactions()

    def groupTransactions(self):

        # reset groups
        self.transactions.loc[:, "groupID"] = "None"

        groups = self.groupSettings.groups

        for key in self.groupSettings.dbIdentifier:
            for grpName, grpList in reversed(groups.items()):
                searchPat = r"(" + r"|".join(grpList) + ")"
                matches = self.transactions[key].str.extractall(
                    searchPat, re.IGNORECASE)

                if matches.empty:
                    continue

                ids = list(matches.index.droplevel(1).values)
                occupiedIds = list(
                    self.transactions.loc[ids, "groupID"] == "None")
                ids = [i for i, b in zip(ids, occupiedIds) if b]
                self.transactions.loc[ids, "groupID"] = grpName



import json
import pprint
from abc import ABC
from pathlib import Path

from econicer.auxiliary import json2Dict

pp = pprint.PrettyPrinter()


class Settings(ABC):

    _settingsName = ""

    def listCurrentSettings(self):
        """Print all attributes"""
        print(f"Current {self._settingsName} settings:")
        out = self.getSettingsDict()
        pp.pprint(out)

    def getSettingsDict(self):
        out = {k: v for k, v in self.__dict__.items() if not k.startswith("_")}

        for k, v in out.items():
            if isinstance(v, Path):
                out[k] = str(v)
        return out

    def write(self):
        out = self.getSettingsDict()

        outPath = Path(".db\\settings.json")
        outPath.parent.mkdir(parents=True, exist_ok=True)
        with open(Path(str(outPath)), "w") as f:
            json.dump(out, f, indent=4)


class DatabaseSettings(Settings):

    _settingsName = "Database Settings"

    delimiter = ";"
    owner = 3
    accountNumber = 4
    bank = 5
    beginTable = 11
    dateFormat = "%d.%m.%Y"
    table = [
        "date",
        "valtua",
        "customer",
        "type",
        "usage",
        "saldo",
        "saldoCurrency",
        "value",
        "valueCurrency",
        "groupID"
    ]


class ExternalSettings(Settings):
    """Abstract class which defines settings behavior"""

    _defaultFile = ""

    def __init__(self, filename="", verbose=True):

        if filename == "":
            self._filename = self._defaultFile
        else:
            self._filename = filename
        settingsPath = Path(self._filename)

        if settingsPath.is_file():

            settDict = json2Dict(settingsPath)

            for k, v in self.__class__.__dict__.items():

                if k.startswith("_"):
                    continue

                if callable(v):
                    continue

                if k in settDict.keys():
                    setattr(self, k, settDict[k])
                else:
                    setattr(self, k, v)
        else:
            if verbose:
                print(
                    f"Could not find settings file for {self._settingsName}. Use default parameters")

            for k, v in self.__class__.__dict__.items():
                # add attribute checking
                if k.startswith("_"):
                    continue

                if callable(v):
                    continue

                setattr(self, k, v)


class EconicerSettings(ExternalSettings):
    """Define parameters for the econicer app"""

    _settingsName = "Econicer"

    currentAccount = ""
    currentAccountFile = ""
    accountList = []
    inputType = Path(r"config\bank.json")
    group = Path(r"config\grouping.json")
    # database = Path(r"config\database.json")
    plotDir = Path("plots")

    def changeAccount(self, accountName, accountFile):
        if accountName == self.currentAccount:
            print(f"Already on {self.currentAccount} account")
        elif accountName in self.accountList:
            self.currentAccount = accountName
            self.currentAccountFile = accountFile
            self.plotDir = str(Path("plots") / self.currentAccount)
            print(f"Set current account to {self.currentAccount}")
        else:
            print(f"Unknown account {accountName}")
            print("Available accounts are:")
            pp.pprint(self.accountList)

    def updateAccountList(self):

        if not self._filename:
            accList = []
        else:
            basePath = Path(self._filename).parent
            accList = [p.stem for p in basePath.iterdir() if p.is_dir()]

        self.accountList = accList

        # files = [x for x in p if x.is_file()]


class GroupSettings(ExternalSettings):
    """Define grouping settings for transaction analysis"""

    _settingsName = "transaction grouping"

    dbIdentifier = []
    groups = {}


class BankFileSettings(ExternalSettings):
    """Define read parameters for transaction file from some Bank"""

    _settingsName = "bank transaction file"

    delimiter = ""
    beginTable = -1
    accountNumber = -1
    bank = -1
    owner = -1
    dateFormat = r""
    table = {}


if __name__ == "__main__":

    sdb = EconicerSettings(".db\\settings.json")
    sdb.updateAccountList()
    sdb.write()
    sdb.listCurrentSettings()
    print(sdb.accountList)

    sdb = EconicerSettings()
    print(sdb.accountList)



import re
import shutil
from pathlib import Path

import numpy as np
import pandas as pd

from econicer.account import BankAccount
from econicer.ecoplot import Ecoplot
from econicer.fileIO import FileIO
from econicer.report import ReportDocument
from econicer.settings import BankFileSettings
from econicer.settings import DatabaseSettings
from econicer.settings import EconicerSettings
from econicer.settings import GroupSettings


def printSum(transactionDataframe):
    print(f"\n Sum of expanses: {transactionDataframe.value.sum():.2f}")


class AccountManager:

    dbFileName = "history.csv"

    def __init__(self, databasePath=".db", settingsPath=".db//settings.json"):

        self.db = Path(databasePath)
        self.settingsPath = Path(settingsPath)

        # load general settings from file
        self.settings = EconicerSettings(self.settingsPath)

        # assign database definition class
        self.dbSettings = DatabaseSettings()

        # load input file settings from file
        self.bankSettings = BankFileSettings(self.settings.inputType)

        self.groupSettings = GroupSettings(self.settings.group)

    def initDB(self, name):
        filepath = self.db / name / self.dbFileName

        if filepath.is_file():
            print(f"Account {name} already exitsts")
            return False

        print(f"Initialize empty account for {name}")
        emptyTransactions = pd.DataFrame(columns=BankAccount.dataframeCols)
        acc = BankAccount(name, None, None, emptyTransactions, {})

        dbFile = FileIO(filepath, self.dbSettings)
        dbFile.writeDB(acc)

        return True

    def update(self, filepath):

        self.makeBackup()

        updateFile = FileIO(filepath, self.bankSettings)
        updateAcc = updateFile.readDB(self.groupSettings)
        # updateAcc.groupTransactions()

        dbFile = FileIO(self.settings.currentAccountFile, self.dbSettings)
        dbAcc = dbFile.readDB(self.groupSettings)

        if not len(dbAcc.accountNumber):
            dbAcc.accountNumber = updateAcc.accountNumber

        if not len(dbAcc.bank):
            dbAcc.bank = updateAcc.bank

        # compare accounts
        if dbAcc.accountNumber != updateAcc.accountNumber:
            print("WARNING! Bank account number is missmatching")

        if dbAcc.bank != updateAcc.bank:
            print("WARNING! Bank institute is missmatching")

        dbAcc.update(updateAcc.transactions)

        dbFile.writeDB(dbAcc)

    def makeBackup(self):
        undoFile = f"{self.settings.currentAccountFile}.old"
        shutil.copy2(self.settings.currentAccountFile, undoFile)

    def undo(self):
        undoFile = f"{self.settings.currentAccountFile}.old"
        shutil.copy2(undoFile, self.settings.currentAccountFile)

    def regroup(self):
        self.makeBackup()

        dbFile = FileIO(self.settings.currentAccountFile, self.dbSettings)
        dbAcc = dbFile.readDB(self.groupSettings)
        dbAcc.groupTransactions()
        dbFile.writeDB(dbAcc)

    def listNoGroups(self, category=None):

        dbFile = FileIO(self.settings.currentAccountFile, self.dbSettings)
        dbAcc = dbFile.readDB(self.groupSettings)

        if category:
            trans = dbAcc.transactions[category[0]]
        else:
            trans = dbAcc.transactions
        noGrp = trans[dbAcc.transactions["groupID"] == "None"]
        if noGrp.empty:
            print("All transactions are grouped.")
        else:
            print(noGrp)

    def listGroup(self, group):
        pd.set_option("display.max_rows", None)

        dbFile = FileIO(self.settings.currentAccountFile, self.dbSettings)
        dbAcc = dbFile.readDB(self.groupSettings)

        transFiltered = dbAcc.transactions[dbAcc.transactions["groupID"] == group]
        print(transFiltered)
        printSum(transFiltered)

    def search(self, search, category):
        keyword = fr"({search})"

        if category is None:
            categories = ["usage"]
        else:
            categories = category

        print(f"Seaching for {search} in {categories}")

        dbFile = FileIO(self.settings.currentAccountFile, self.dbSettings)
        dbAcc = dbFile.readDB(self.groupSettings)

        ids = []
        for cat in categories:
            subDF = dbAcc.transactions[cat]
            matches = subDF.str.extractall(keyword, re.IGNORECASE)
            if not matches.empty:
                tmp = list(matches.index.droplevel(1).values)
                ids = ids + tmp
        if ids:
            ids = np.unique(ids)
            trans = dbAcc.transactions.loc[ids, :]
            print(trans)
            printSum(trans)
        else:
            print("Could not find any matches")

    def createPlots(self):

        dbFile = FileIO(self.settings.currentAccountFile, self.dbSettings)
        dbAcc = dbFile.readDB(self.groupSettings)

        plotDir = Path(self.settings.plotDir)
        if not plotDir.exists():
            plotDir.mkdir(parents=True)

        transactions = dbAcc.transactions

        ep = Ecoplot(str(plotDir))
        ep.plotTimeline(transactions)
        ep.plotPie(transactions)
        ep.plotBars(transactions)
        ep.plotCategories(transactions)
        ep.plotBarsYearly(transactions)
        ep.plotCategoriesYearly(transactions)

        self.plotPaths = ep.plotPaths

    def createReport(self):

        self.createPlots()

        dbFile = FileIO(self.settings.currentAccountFile, self.dbSettings)
        dbAcc = dbFile.readDB(self.groupSettings)

        rp = ReportDocument(dbAcc.owner, dbAcc.accountNumber, dbAcc.bank)
        rp.addOverallSection(self.plotPaths["all"])
        rp.addYearlyReports(self.plotPaths["years"])
        rp.generatePDF()



import argparse
import sys
from pathlib import Path

from econicer.accountManager import AccountManager
from econicer.settings import EconicerSettings


def main():

    parser = argparse.ArgumentParser(
        description=""
        "   ___  _________  ____  __________  ____\n"
        "  / _ \/ ___/ __ \/ __ \/ / ___/ _ \/ __/\n"
        " /  __/ /__/ /_/ / / / / / /__/  __/ /\n"
        " \___/\___/\____/_/ /_/_/\___/\___/_/\n\n"
        " a perception of economic success",
        formatter_class=argparse.RawDescriptionHelpFormatter)

    parser.add_argument(
        "-i",
        "--init",
        metavar="NAME",
        help="initialize new account",
        default=""
    )
    parser.add_argument(
        "-c",
        "--change",
        metavar="NAME",
        help="change account or create new",
        default=""
    )
    parser.add_argument(
        "-a",
        "--add",
        metavar="FILE",
        help="add to current account"
    )
    parser.add_argument(
        "-s",
        "--search",
        metavar="KEYWORD",
        help="Search for specified keyword in category 'usage' and lists sum of expanses",
        default=""
    )
    parser.add_argument(
        "-k",
        "--category",
        metavar="CATEGORY",
        help="Categories to search in",
        nargs="+"
    )
    parser.add_argument(
        "-l",
        "--listGroup",
        metavar="GROUP",
        help="display current settings"
    )
    parser.add_argument(
        "-ls",
        "--listSettings",
        help="display current settings",
        action="store_true"
    )
    parser.add_argument(
        "-n",
        "--listNoGroup",
        help="List all transactions without a group",
        action="store_true"
    )
    parser.add_argument(
        "-u",
        "--undo",
        help="Undo last change on database",
        action="store_true"
    )
    parser.add_argument(
        "-g",
        "--group",
        help="regroup database",
        action="store_true"
    )
    parser.add_argument(
        "-p",
        "--plot",
        help="make plots",
        action="store_true"
    )
    parser.add_argument(
        "-r",
        "--report",
        help="automated report",
        action="store_true"
    )

    args = parser.parse_args()

    if len(sys.argv) == 1:
        # display help message when no args are passed.
        parser.print_help()
        sys.exit()

    # Path objects to database and settings file
    db = Path.cwd() / ".db"
    settingsPath = Path(db / "settings.json")

    # check if db folder exists
    if not db.is_dir():
        print("No database folder found. Creating new database folder")
        db.mkdir(parents=True)

    if not settingsPath.is_file():
        print("No database settings file found. Creating default settings")
        ecoSettings = EconicerSettings(verbose=False)
        ecoSettings.write()

        print("Please edit the settings file to your preferences")

    ecoSettings = EconicerSettings(settingsPath, verbose=False)

    # list current settings
    if args.listSettings:
        ecoSettings.listCurrentSettings()
        exit()

    # change settings
    if args.change:
        accPath = db / "//".join([args.change, AccountManager.dbFileName])
        ecoSettings.changeAccount(args.change, accPath)
        ecoSettings.write()
        exit()

    # init new account
    if args.init:
        accountMan = AccountManager()
        initSuccess = accountMan.initDB(args.init)

        if initSuccess:
            ecoSettings.updateAccountList()
            accPath = db / "//".join([args.init, AccountManager.dbFileName])
            ecoSettings.changeAccount(args.init, accPath)
            ecoSettings.write()
        exit()

    if args.undo:
        accountMan = AccountManager()
        accountMan.undo()
        exit()

    # Add file to account history
    if args.add:
        accountMan = AccountManager()
        accountMan.update(args.add)
        exit()

    # regroup database
    if args.group:
        accountMan = AccountManager()
        accountMan.regroup()
        exit()

    # list all transactions in current account without group
    if args.listNoGroup:
        accountMan = AccountManager()
        accountMan.listNoGroups(args.category)
        exit()

    # list all transactions in current account without group
    if args.listGroup:
        accountMan = AccountManager()
        accountMan.listGroup(args.listGroup)
        exit()

    # search for keyword in specified categories
    if args.search:
        accountMan = AccountManager()
        accountMan.search(args.search, args.category)
        exit()

    # Create plots from current history
    if args.plot:
        accountMan = AccountManager()
        accountMan.createPlots()
        exit()

    # print report for all account data
    if args.report:
        accountMan = AccountManager()
        accountMan.createReport()
        exit()


if __name__ == "__main__":
    main()



import csv
import datetime
from pathlib import Path

import pandas as pd

from econicer.auxiliary import str2num
from econicer.account import BankAccount


class FileIO:

    def __init__(self, filepath, settings, str2numConversion=True):

        self.filepath = filepath
        self.settings = settings
        self.str2numConversion = str2numConversion

    def readHeader(self):
        """extract header account information from database"""
        with open(self.filepath) as csvFile:
            header = [next(csvFile)
                      for x in range(self.settings.beginTable)]

        owner = header[self.settings.owner].split(
            self.settings.delimiter)[1].replace("\n", "")

        accountNumber = header[self.settings.accountNumber].split(
            self.settings.delimiter)[1].replace("\n", "")

        bank = header[self.settings.bank].split(
            self.settings.delimiter)[1].replace("\n", "")

        return owner, accountNumber, bank

    def readBody(self):

        transactionDF = pd.read_csv(
            self.filepath,
            sep=self.settings.delimiter,
            header=self.settings.beginTable,
            skip_blank_lines=False
        )

        transactionDF["date"] = pd.to_datetime(
            transactionDF["date"], format=self.settings.dateFormat)

        transactionDF["valtua"] = pd.to_datetime(
            transactionDF["valtua"], format=self.settings.dateFormat)

        if self.str2numConversion:
            transactionDF["value"] = transactionDF["value"].apply(str2num)
            transactionDF["saldo"] = transactionDF["saldo"].apply(str2num)

        return transactionDF

    def readDB(self, groupSettings):
        owner, accountNumber, bank = self.readHeader()
        transactionDF = self.readBody()

        return BankAccount(owner, accountNumber, bank, transactionDF, groupSettings)

    def writeDB(self, account):
        """Write all account inforrmation to database"""

        filepath = Path(self.filepath)
        filepath.parent.mkdir(parents=True, exist_ok=True)

        with open(filepath, "w", newline="") as csvfile:
            csvwriter = csv.writer(
                csvfile,
                delimiter=self.settings.delimiter,
                quotechar="'",
                quoting=csv.QUOTE_MINIMAL
            )

            # Write header
            csvwriter.writerow(["##ECONICER DATABASE"])
            csvwriter.writerow([
                datetime.datetime.now().strftime(
                    "File created at %Y-%m-%d %H:%M:%S")
            ])
            csvwriter.writerow(["#GENERALINFO"])
            csvwriter.writerow(["owner", account.owner])
            csvwriter.writerow(["account number", account.accountNumber])
            csvwriter.writerow(["bank", account.bank])
            csvwriter.writerow(["#STATS"])
            csvwriter.writerow(["totalSum", "..."])
            csvwriter.writerow(["expanseGroupNames", "..."])
            csvwriter.writerow(["expanseGroupValues", "..."])
            csvwriter.writerow(["#TRANSACTIONS"])

        # write table
        account.transactions.to_csv(
            filepath,
            mode='a',
            sep=self.settings.delimiter,
            index=False,
            date_format=self.settings.dateFormat
        )



from setuptools import setup
from setuptools import find_packages


with open("README.md", "r", encoding="utf-8") as fh:
    long_description = fh.read()

setup(
    name="econicer",
    version="0.23",
    author="spiony",
    author_email="glumt@protonmail.com",
    description="A small program to analyse financial transactions",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/Spiony/econicer",
    projects_urls={
        "Bug Tracker": "https://github.com/Spiony/econicer/issues",
    },
    classifiers=[
        "Programming Language :: Python :: 3",
        "Operating System :: OS Independent",
        "License :: OSI Approved :: GNU General Public License v3 (GPLv3)"
    ],
    packages=find_packages(),
    python_requires=">=3.8",
    install_requires=[
        'matplotlib>=3.4.1',
        'PyLaTeX>=1.4.1',
        'pandas>=1.2.3',
    ],
    include_package_data=True,
)



import json
import datetime
import numpy as np


def nextMonth(date):
    """returns datetime object of first day of next month"""
    if date.month <= 11:
        firstDayNextMonth = datetime.datetime(date.year, date.month + 1, 1)
    else:
        firstDayNextMonth = datetime.datetime(date.year + 1, 1, 1)
    return np.datetime64(firstDayNextMonth), firstDayNextMonth


def endOfMonth(date):
    """returns datetime object of last day of next month"""
    _, firstDayNextMonth = nextMonth(date)
    lastDayOfMonth = datetime.datetime.combine(
        firstDayNextMonth - datetime.timedelta(1),
        datetime.datetime.min.time())
    return np.datetime64(lastDayOfMonth), lastDayOfMonth


def nextYear(date):
    """returns datetime object of beginning of next year"""
    firstDayNextYear = datetime.datetime.combine(
        datetime.datetime(date.year + 1, 1, 1), datetime.datetime.min.time())
    return np.datetime64(firstDayNextYear), firstDayNextYear


def endOfYear(date):
    """returns datetime object for the end of this year"""
    lastDayOfYear = datetime.datetime.combine(
        datetime.datetime(date.year, 12, 31), datetime.datetime.min.time())
    return np.datetime64(lastDayOfYear), lastDayOfYear


def str2num(stringInput):

    if isinstance(stringInput, float):
        return stringInput

    if isinstance(stringInput, str):
        if "." in stringInput and "," in stringInput:
            stringInput = stringInput.replace('.', '')
            stringInput = stringInput.replace(',', '.')
            return float(stringInput)
        elif "," in stringInput:
            stringInput = stringInput.replace(',', '.')
            return float(stringInput)
        else:
            return float(stringInput)


def json2Dict(filepath):
    with open(str(filepath), 'r', encoding='utf-8') as jsonFile:
        jsonContent = jsonFile.read()
    return json.loads(jsonContent)



import pylatex as tex


class ReportDocument():

    def __init__(self, name, number, bank):
        self.name = name
        self.number = number
        self.bank = bank

        geometry_options = {
            "tmargin": "35mm",
            "lmargin": "25mm",
            "textwidth": "160mm",
            "textheight": "237mm",
        }

        self.doc = tex.Document(
            f"AccountReport_{self.name}",
            documentclass="article",
            document_options=['10pt', "a4paper"],
            geometry_options=geometry_options,
            lmodern=False
        )
        self.doc.preamble.append(tex.NoEscape(
            r"\renewcommand{\familydefault}{\sfdefault}"))
        self.doc.preamble.append(tex.Command('usepackage', 'helvet'))
        self.doc.preamble.append(tex.Command(
            'usepackage', arguments='placeins', options="section"))

        self.addHeader()

        self.doc.preamble.append(tex.Command('title', "Financial Report"))
        # self.doc.preamble.append(Command('bank', 'Anonymous author'))
        self.doc.preamble.append(tex.Command('date', tex.NoEscape(r'\today')))
        # \usepackage[section]{placeins}

    def generatePDF(self):
        self.doc.generate_pdf(compiler="xelatex", clean_tex=True)

    def addHeader(self):
        # Add document header
        header = tex.PageStyle(
            "header", header_thickness=1, footer_thickness=1)
        # Create left header
        with header.create(tex.Head("L")):
            header.append(f"Account name: {self.name}")
            header.append(tex.LineBreak())
            header.append(f"IBAN: {self.number}")

        # Create right header
        with header.create(tex.Head("R")):
            header.append(self.bank)

        # Create left footer
        with header.create(tex.Foot("L")):
            header.append("Econicer - Financial Report")

        # Create right footer
        with header.create(tex.Foot("R")):
            header.append("Page ")
            header.append(tex.Command("thepage"))

        self.doc.preamble.append(header)
        self.doc.change_document_style("header")

    def addOverallSection(self, plotPaths):

        with self.doc.create(tex.Section('Overall Financial Report')):
            self.doc.append('Report for all available data')
            with self.doc.create(tex.Figure(position='h!')):
                with self.doc.create(tex.SubFigure(
                        position='b',
                        width=tex.NoEscape(r'0.5\linewidth'))) as timeLine:

                    timeLine.add_image(
                        str(plotPaths["timeline"]),
                        width=tex.NoEscape(r'\linewidth')
                    )
                    timeLine.add_caption("Account saldo timeline")

                with self.doc.create(tex.SubFigure(
                        position='b',
                        width=tex.NoEscape(r'0.5\linewidth'))) as pie:

                    pie.add_image(
                        str(plotPaths["pie"]),
                        width=tex.NoEscape(r'\linewidth')
                    )
                    pie.add_caption('Cash flow distribution by category')

                self.doc.append(tex.LineBreak())

                with self.doc.create(tex.SubFigure(
                        position='b',
                        width=tex.NoEscape(r'0.5\linewidth'))) as pie:

                    pie.add_image(
                        str(plotPaths["years"]),
                        width=tex.NoEscape(r'\linewidth')
                    )
                    pie.add_caption('Yearly income and expanses')
                with self.doc.create(tex.SubFigure(
                        position='b',
                        width=tex.NoEscape(r'0.5\linewidth'))) as pie:

                    pie.add_image(
                        str(plotPaths["categories"]),
                        width=tex.NoEscape(r'\linewidth')
                    )
                    pie.add_caption(
                        "Summation of expanses by category for all years")
        self.doc.append(tex.Command("newpage"))

    def addYearlyReports(self, plotPaths):

        for year, paths in plotPaths.items():
            self.addYearSection(year, paths)

    def addYearSection(self, year, plotPaths):

        with self.doc.create(tex.Section(f'Financial Reprot {year}')):
            with self.doc.create(tex.Figure(position='h!')):
                with self.doc.create(tex.SubFigure(
                        position='b',
                        width=tex.NoEscape(r'0.5\linewidth'))) as pie:

                    pie.add_image(
                        str(plotPaths["year"]),
                        width=tex.NoEscape(r'\linewidth')
                    )
                    pie.add_caption('Monthly income and expanses')
                with self.doc.create(tex.SubFigure(
                        position='b',
                        width=tex.NoEscape(r'0.5\linewidth'))) as pie:

                    pie.add_image(
                        str(plotPaths["categories"]),
                        width=tex.NoEscape(r'\linewidth')
                    )
                    pie.add_caption(
                        "Summation of expanses by category for this year")
